* ------------------------ BEGIN DISTAZDP ----------------------
      subroutine distazdp( wlongsign, alat, along, blat, blong,
     * rdeg, rkm, az, baz)
c 
c compute distances, azimuths using formulas from
c Bruce Julian.
c
c Dates: 01/27/84 - written by D. Boore
*        06/02/98 - Converted to double precision
*        10/19/01 - Changed name to DISTAZDP
c

      double precision alat, along, blat, blong, 
     : alatr, alongr, blatr, blongr,
     : a, b, ca, cb, sa, sb, sd, cd

      pi = 4.0 * atan( 1. )
      dtor = pi/ 180.
c
c convert from degrees to radians and correct sign of
c longitude so that east longitude is positive.
c
      alatr = dtor * alat
      alongr = -dtor * along * wlongsign
      blatr = dtor * blat
      blongr = -dtor * blong * wlongsign
c
c compute geocentric latitudes.
c
      alatr = datan( dble(0.993305) * dtan( alatr ) )
      blatr = datan( dble(0.993305) * dtan( blatr ) )
c
c compute latitude dependent quantities
c
      ca = dcos( alatr )
      cb = dcos( blatr )
      sa = dsin( alatr )
      sb = dsin( blatr )
c
c now compute other quantities
c
      a = cb * dsin( blongr - alongr )
      b = ca * sb - sa * cb * dcos( blongr - alongr )
      cd = ca * cb * dcos( blongr - alongr ) + sa * sb
      sd = dsqrt( a*a + b*b )
c
c compute distances
c
      rdeg = datan2( sd, cd )/ dtor
      rkm = 111.19 * rdeg
c
c compute azimuth (from a to b) and make it positive.
c
      az = datan2( a, b )/ dtor
      if ( az .lt. 0.0 ) az = az + 360.0
c
c compute back azimuth (from b to a) and make it positive.
c
      a = ca * dsin( alongr - blongr )
      b = cb * sa - sb * ca * dcos( alongr - blongr )
      baz = datan2( a, b)/ dtor
      if ( baz .lt. 0.0 ) baz = baz + 360.0
c
      return
      end
* ------------------------ END DISTAZDP ----------------------
