      Program Km2Deg

! Computes lat and long for a specified distance in km N and E of a
! reference point

! If want coords corresponding to a distance D and azimuth AZ from a reference
! point, use this program with N = D*cos(AZ) and E = D*sin(AZ)

! Sample input (col. 1 starts after the initial "*")

!Name of output file:
!  km2deg.out
!Coordinates (title, ref. coord. (+east), n, e dist to point (km)):
!  JAB
! 34.31149 -118.49675  Ref for JAB
! 0.00250  0.00963    dist to JAB
!  JGB 
! 34.31295 -118.49890  Ref for JGB
! 0.00395  0.00588    dist to JGB
! stop


! Dates: 05/27/98 - Written by D.M. Boore
!        06/01/98 - Changed to double precision
!        06/03/98 - Obtain information from a control file
!        09/16/00 - Made east long positive, added simple computation
!                   for coordinates
!        04/30 02 - Suggest name of control file
!        02/26/04 - Cleanup for use in LF95
!        02/14/09 - Change input to km2deg_f to single
!                   precision, and convert to double precision
!                   in the subprogram
!        11/18/15 - Some modernization of code

      character buf*70, f_in*30, f_out*30, buf4*4
!      double precision vn, ve, alat_ref, along_ref, vlat, vlong

      logical f_in_exist

      f_in_exist = .false.
      do while (.not. f_in_exist)
        f_in = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = KM2DEG.CTL;'//
     :    ' CTL-BRK to quit): '
        read(*, '(a)') f_in
        if (f_in(1:4) == '    ') f_in = 'km2deg.ctl'
        call trim_c(f_in,nc_f_in)
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_in)
      open(nu_in, file=f_in(1:nc_f_in), status = 'unknown')
      write(*, '(1x,a)') f_in(1:nc_f_in)

! Obtain info from control file:

      read(nu_in,*)                            ! skip comment line
      f_out = ' '
      read(nu_in, '(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(nu_out, file=f_out(1:nc_f_out), status = 'unknown')
      
      read(nu_in,*)                            ! skip comment line

      loop_over_input: DO

        buf = ' '
        read(nu_in, '(a)') buf                   ! read title or stop
        call trim_c(buf,nc_buf)
        write(*, '(1x,a)') buf(1:nc_buf)
        write(nu_out,*) 
        write(nu_out,'(a)') buf(1:nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(buf4(1:4) == 'STOP') then
          close(nu_in)
          close(nu_out)
          stop
        end if
      
        read(nu_in,*) alat_ref, along_ref        ! reference coordinates
        read(nu_in,*) vn, ve                     ! dist. to point

        call km2deg_f( vn, ve, alat_ref, along_ref, 
     :               vlat, vlong )

        write(*,'(a,2(1x,f10.5))') ' vlat, vlong = ', vlat, vlong

        write(nu_out,'(a,2(1x,f10.5))') ' Ref. lat, long. = ',
     :   alat_ref, along_ref

        write(nu_out,'(a,2(1x,f10.5))') ' Distance north, east (km) = ',
     :   vn, ve

      write(nu_out,'(a,2(1x,f10.5))') 
     :  ' Using Richter tables: vlat, vlong = ', vlat, vlong

        pi = 4.0 * atan(1.0)
        d2r = pi/180.0

        vlat = alat_ref + vn/(6371.0 * d2r)
        vlong = along_ref + ve/(6371.0 * d2r * 
     :          cos(0.5 * (alat_ref + vlat) * d2r))
        write(nu_out,'(a,2(1x,f10.5))') 
     :    ' Using sphere approx : vlat, vlong = ', vlat, vlong

      END DO loop_over_input

      end

      include '\forprogs\km2deg_f.for'
      include '\forprogs\locate_d.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\get_lun.for'

