      Program dist_3d

! Uses a control file; here is an example:
!!Control file for program Dist_3D
!!Revision of program involving a change in the control file on this date:
!   12/20/19
!!As many comment lines as desired, each starting with "!"
!!
!!Name of output file:
! dist_3d.out
!!Minimum Depth for Campbell:
! 3.0
!!Number of Fault Segments:
! 3
!!Are reference point and station coordinates in Cartesian coordinates (Y/N)?
! N
!!  Enter coordinates in this order
!!  if Y: N, E coordinates from some arbitrary origin (not necessarily the reference point,
!!        which can be different for each fault segment)
!!  if N: latitude, longitude (positive to the east)
!!  NOTE: The coordinates are referred to as "N" and "E" below, even if the coordinates
!!  are latitude and longitude.  I was going to use "X" and "Y", but those variables
!!  are already used in the programs to mean different things, and I did not want
!!  to add to the confusion.  Besides, "N" and "E" have the same general meaning in terms of
!!  direction from an origin to a point for both Cartesian and geographic coordinates.
!!
!!Reference Point and Fault Orientation (ref N, E, elev(m), depth, strike, dip, s1, s2, w1, w2):
!! depth, s1, s2, w1, w2 have units of kilometers
!! NOTE this requirement on the reference point coordinates: it must be on the
!!   planar surface containing the rectangular area that is assumed to have ruptured.
!!   The reference point need not be on the ruptured surface, although for simplicity,
!!   I recommend that it be some well defined point on the rupture surface, such as the hypocenter
!!   or one of the corners of the rectangle approximating the rupture surface.
!! NOTE this requirement on the fault dimensions: s1<s2, w1<w2
!! s1 and s2 are determined by looking along strike from the reference point.
!! w1 and w2 are distances updip and downdip from the reference point.
! 23.61719 120.68854 0.0 0.94 3.0 29.0 0.0 55.0 0.0 40.0
! 24.11315 120.71685 0.0 0.94 3.0 29.0 0.0 25.0 0.0 12.5
! 24.10797 120.82414 0.0 7.00 3.0  5.0 0.0 25.0 0.0 27.5 
!!Station Location (N, E, elev (m), character string for output (can be blank):
! 24.00000 120.83333 0.0   in seg A
! 24.40000 120.60000 0.0   upper left corner
! 24.21666 121.00000 0.0   in seg C
! 23.83333 121.20000 0.0   halfway up and east of seg A
! 24.25000 120.78333 0.0   in seg B
! 24.25000 120.71666 0.0   hanging wall side of seg B
!stop

! Dates: 12/07/98 - Written by D. Boore
!        09/17/00 - Allow for multiple faults, use subroutine dist3dmf
!                   to call dist_3df, change control file
!        12/03/00 - Change in control file, and include ref and sta elevation
!        04/30/02 - Suggest name of control file
!        09/16/03 - Add "stop" to control file
!        09/27/05 - Increase size of f_out
!        05/20/09 - Add "!" before control comment lines
!                 - Compute and print coords of surface projections of vertices
!                   based on program prjctflt.for (Not here, because
!                   I found fltprjct.for that uses a control file---but
!                   it does not include ref_elev, so I should modify it;
!                   someday I should include the projection computations here).
!        08/23/10 - Write character string to output
!        09/16/10 - Modified dist_3df to compute az_jb, but write a note
!                   indicating that this is useful only for a single-segment model.
!        09/27/10 - Reposition sta_info field and write out distances and 
!                   az_jb for each fault segment.
!        06/22/13 - Added check of relative size of s1,s2 and w1,w2 (this arose because
!                   Andreas Skarlatoudis was getting result in disagreement with those
!                   from a program by Bob Youngs for some Greek events/stations).
!        12/06/16 - Add horizontal and slant distances from the reference location to
!                   station.
!        01/10/17 - In writing name of control file in output file, correct 
!                   "f_ctl(1:nc_f_out)" to "f_ctl(1:nc_f_ctl)"
!                 - Added rx_nga to output
!        12/17/19 - Add ry0_nga to the output
!        12/20/19 - Allow reference and station coordinates to be in Cartesian 
!                   coordinates or geographic coordinates (lat/long)
!                 - Revise for new ctl file, and add more to output file

      implicit none

      real h_min_c, elev_sta, E_sta, N_sta, 
     :     r_ref2sta_horiz, sign_west_longitude, baz, az, rdeg,   
     :     r_ref2sta_slant, az_c, d_c, az_cd2f, d_cd2f, ry0_nga, 
     :     rx_nga, az_jb, d_jb 

      integer nc_f_ctl, nu_ctl, nc_cmnts2skip, nc_date_ctl_in,
     :  nc_date_ctl_correct, nc_f_out, nu_out, n_f, nc_cstring, 
     :  i, nc_colheads, ista, nc_buf, ISTAT, iend, ibgn, icase_jb 

      character date_ctl_correct*8, date_ctl_in*30
      character f_ctl*60
      character cmnts2skip(500)*200
     
      character buf*200, f_out*140, buf4*4, cstring*60
      real N_ref(10), E_ref(10), elev_ref(10), h_ref(10), 
     :     strike_f(10), dip_f(10), s1(10), s2(10), w1(10), w2(10),
     :     d_jb_segs(10), d_cd2f_segs(10), d_c_segs(10),
     :     az_jb_segs(10), rx_nga_segs(10), ry0_nga_segs(10)
     
      character colheads*410
     
      integer icase_jb_segs(10)
     
      integer status

      logical f_ctl_exist, cartesian_coords

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (enter = DIST_3D.CTL;'//
     :    ' CTL-BRK to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'dist_3d.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '12/20/19'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; '//
     :       'the proper date is '//
     :        date_ctl_correct(1:nc_date_ctl_correct)
        write(*,'(a)') 'Update your control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_out,'(2a)') 
     :  ' Output of program Dist_3D, using control file ', 
     :    f_ctl(1:nc_f_ctl)

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      call write_cmnts2skip(nu_out, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) h_min_c
      write(nu_out,*) h_min_c

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      call write_cmnts2skip(nu_out, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) n_f
      write(nu_out,*) n_f

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      call write_cmnts2skip(nu_out, cmnts2skip, nc_cmnts2skip)
      cstring = ' '
      read(nu_ctl,'(a)') cstring
      call trim_c(cstring,nc_cstring)
      call upstr(cstring)
      if (cstring(1:1) == 'Y') then
        cartesian_coords = .true.
      else
        cartesian_coords = .false.
      end if
      write(nu_out,'(1x,a)') cstring(1:1)

      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      call write_cmnts2skip(nu_out, cmnts2skip, nc_cmnts2skip)
      write(nu_out,'(a)') '!Reference Point and Fault orientation'
      write(nu_out,'(5x,a, 6x,a, 1x,a, 1x,a,   
     :               2x,a, 5x,a, 
     :               6x,a, 6x,a, 6x,a, 6x,a)')
     : 'ref_N', 'ref_E', 'ref_elev', 'ref_depth',
     : 'strike', 'dip', 
     : 's1', 's2', 'w1', 'w2'
      do i = 1, n_f
        read(nu_ctl,*) N_ref(i), E_ref(i), elev_ref(i), h_ref(i),
     :                 strike_f(i), dip_f(i), 
     :                 s1(i), s2(i), w1(i), w2(i)
     
        if (s1(i) >= s2(i) ) then
          print *, ' i, s1, s2 = ', i, s1(i), s2(i)
          print *, ' s1 >= s2; this is not allowed; QUITTING!'
          close(nu_out)
          close(nu_ctl)
          stop
        end if
        if (w1(i) >= w2(i) ) then
          print *, ' i, w1, w2 = ', i, w1(i), w2(i)
          print *, ' w1 >= w2; this is not allowed; QUITTING!'
          close(nu_out)
          close(nu_ctl)
          stop
        end if
        
        write(nu_out,'(1x,f9.5, 1x,f10.5, 2x,f7.1, 3x,f7.2,
     :                 6(1x,f7.2))') 
     :                 N_ref(i), E_ref(i), elev_ref(i), h_ref(i),
     :                 strike_f(i), dip_f(i), 
     :                 s1(i), s2(i), w1(i), w2(i)
     
      end do

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
 
      write(nu_out,'(a)') '!'
      write(nu_out,'(a)') '!Results'
      write(nu_out,'(a)') '!'
      write(nu_out,'(a)') '!  The results for one station per line.'
      write(nu_out,'(a)') 
     :  '!  Following the station information are distances'//
     :  '!  for all of the fault segments, computed as the minimum'
      write(nu_out,'(a)') 
     :  '!  over all fault segments of the same distance measure '//
     :     'for each segment.'
      write(nu_out,'(a)') 
     :  '!  This is followed by results for each fault segment,'
      write(nu_out,'(a)') 
     :  '!  including the icase defined in Figure 4 of Kaklamanos'
      write(nu_out,'(a)') 
     :  '!  et al.,2011, Earthq.Spectra 27, p. 1219-1235.'
      write(nu_out,'(a)') 
     :  '!  icase can be used to determine if a '//
     :     'station is on the hanging or footwall '//
     :     'side of the fault:'
      write(nu_out,'(a)') 
     :  '!    foot wall: icase = 1, 4, or 7'
      write(nu_out,'(a)') 
     :  '!    hanging wall: icase = 2, 3, 5, 6, 8, or 9'
      write(nu_out,'(a)') 
     :  '!  az_jb (defined in Kaklamanos et al., '//
     :     'with "alpha" as the variable name), '//
     :     'is an easier way to determine this:'
      write(nu_out,'(a)') 
     :  '!    foot wall: az_jb < 0'
      write(nu_out,'(a)') 
     :  '!    hanging wall: az_jb < 0'
      write(nu_out,'(a)') 
     :  '!  rx_nga is another way to determine this'//
     :                         ' (and useful on some recent GMPEs):'
      write(nu_out,'(a)') 
     :  '!    foot wall: rx_nga < 0'
      write(nu_out,'(a)') 
     :  '!    hanging wall: rx_nga < 0'
      write(nu_out,'(a)') 
     :  '!  az_jb is defined as alpha in Kaklamanos et al.'
      write(nu_out,'(a)') 
     :  '!  ry0_nga is used to taper the Abrahamson et al. '//
     :     'hanging wall effect; see p. 1040 of Abrahamson et al., '//
     :     'EqSpectra 30, 1025-1055.'

      colheads = ' '
      call trim_c(colheads,nc_colheads)

      do i = 1, n_f
      
        colheads = colheads(1:nc_colheads)//' fault__:' 
        if (i < 10) then
          write(colheads(nc_colheads+8:nc_colheads+8),'(i1)') i
        else
          write(colheads(nc_colheads+7:nc_colheads+8),'(i2)') i
        end if
        
        call trim_c(colheads,nc_colheads)
      
        colheads = colheads(1:nc_colheads)//
     :    'r_rup r_cmpbl    r_jb'//
     :    ' rx_nga_segs ry0_nga_segs'//
     :    '   az_jb icase'
        call trim_c(colheads,nc_colheads)
      end do
      
      
      write(nu_out,'(a)') '!'
      write(nu_out,'(1x,a, 
     :               53x,a,
     :               5x,a, 6x,a, 1x,a, 
     :               1x,a, 1x,a, 
     :               3x,a, 1x,a, 4x,a, 
     :               2x,a, 1x,a,
     :               2x,a)') 
     :     'sta_num', 
     :     'sta_info',
     :     'sta_N', 'sta_E', 'sta_elev', 
     :     'r_ref2sta_horiz', 'r_ref2sta_slant',  
     :     'r_rup', 'r_cmpbl', 'r_jb', 
     :     'rx_nga', 'ry0_nga',
     :     colheads(1:nc_colheads)
     
      ista = 0

      loop over sites: DO

        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if (status /= 0) then
          exit loop over sites
        end if
        call trim_c(buf, nc_buf)
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4(1:4) .eq. 'STOP') then
          exit loop over sites
        end if
      
      
        ista = ista+1
        
        read(buf(1:nc_buf),*) N_sta, E_sta, elev_sta
        
        call RCF (1,buf,nc_buf,N_sta,ibgn,iend, ISTAT)
        call RCF (2,buf,nc_buf,E_sta,ibgn,iend, ISTAT)
        call RCF (3,buf,nc_buf,elev_sta,ibgn,iend, ISTAT)
        
        cstring = ' '
        cstring = buf(iend+1:nc_buf)
        call trim_c(cstring, nc_cstring)

        if (cartesian_coords) then
          r_ref2sta_horiz = 
     :       sqrt((N_sta-N_ref(1))**2 + (E_sta-E_ref(1))**2)
        else
          sign_west_longitude = -1.0
          call distaz( sign_west_longitude, N_ref(1), E_ref(1), 
     :                     N_sta, E_sta, 
     :                     rdeg, r_ref2sta_horiz, az, baz)
        end if
        r_ref2sta_slant = sqrt(r_ref2sta_horiz**2 + h_ref(1)**2)

        call dist3dmf(cartesian_coords, N_sta, E_sta, elev_sta,
     :   N_ref, E_ref, elev_ref, h_ref, strike_f, dip_f,
     :   w1, w2, s1, s2, n_f, h_min_c, 
     :   icase_jb, d_jb, az_jb, rx_nga, ry0_nga,
     :   d_cd2f, az_cd2f, 
     :   d_c, az_c, d_jb_segs, d_cd2f_segs, d_c_segs,
     :   az_jb_segs, icase_jb_segs, rx_nga_segs, ry0_nga_segs)

      
        write(nu_out, '(4x,i4, 
     :                1x,a60,
     :                1x,f9.5, 1x,f10.5, 2x,f7.1, 
     :                9x,f7.2, 9x,f7.2,
     :                3(1x,f7.2), 
     :                2(1x,f7.2), 
     :                1x,
     :                10(7x,f7.2, 1x,f7.2, 1x,f7.2, 
     :                   5x,f7.2, 6x,f7.2, 
     :                   1x,f7.2, 5x,i1)
     :                                )') 
     :    ista, 
     :    cstring(1:nc_cstring),
     :    N_sta, E_sta, elev_sta,
     :    r_ref2sta_horiz, r_ref2sta_slant,
     :    d_cd2f, d_c, d_jb, 
     :    rx_nga, ry0_nga,
     :    (d_cd2f_segs(i), d_c_segs(i), d_jb_segs(i), 
     :     rx_nga_segs(i), ry0_nga_segs(i),
     :     az_jb_segs(i), icase_jb_segs(i), i = 1, n_f)

      END DO loop over sites

      close(unit=nu_ctl)
      close(unit=nu_out)

      stop
      end

      include '\forprogs\distaz.for'
      include '\forprogs\dist3dmf.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\deg2km_f.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\write_cmnts2skip.for'
      include '\forprogs\locate.for'
      include '\forprogs\rc_subs.for'
